@@ -1,18 +1,16 @@
 import { Menu } from '@grafana/ui';
 import { useAppNotification } from 'app/core/copy/appNotification';
-import {
-  isGrafanaRulerRule,
-  isGrafanaRulerRulePaused,
-  getRuleGroupLocationFromCombinedRule,
-} from 'app/features/alerting/unified/utils/rules';
-import { CombinedRule } from 'app/types/unified-alerting';
+import { GrafanaRuleGroupIdentifier } from 'app/types/unified-alerting';
+import { RulerGrafanaRuleDTO } from 'app/types/unified-alerting-dto';
 
 import { usePauseRuleInGroup } from '../hooks/ruleGroup/usePauseAlertRule';
 import { isLoading } from '../hooks/useAsync';
 import { stringifyErrorLike } from '../utils/misc';
+import { isPausedRule } from '../utils/rules';
 
 interface Props {
-  rule: CombinedRule;
+  rule: RulerGrafanaRuleDTO;
+  groupIdentifier: GrafanaRuleGroupIdentifier;
   /**
    * Method invoked after the request to change the paused state has completed
    */
@@ -23,27 +21,20 @@ interface Props {
  * Menu item to display correct text for pausing/resuming an alert,
  * and triggering API call to do so
  */
-const MenuItemPauseRule = ({ rule, onPauseChange }: Props) => {
+const MenuItemPauseRule = ({ rule, groupIdentifier, onPauseChange }: Props) => {
   const notifyApp = useAppNotification();
   const [pauseRule, updateState] = usePauseRuleInGroup();
 
-  const isPaused = isGrafanaRulerRule(rule.rulerRule) && isGrafanaRulerRulePaused(rule.rulerRule);
-  const icon = isPaused ? 'play' : 'pause';
-  const title = isPaused ? 'Resume evaluation' : 'Pause evaluation';
+  const [icon, title] = isPausedRule(rule)
+    ? ['play' as const, 'Resume evaluation']
+    : ['pause' as const, 'Pause evaluation'];
 
   /**
    * Triggers API call to update the current rule to the new `is_paused` state
    */
   const setRulePause = async (newIsPaused: boolean) => {
-    if (!isGrafanaRulerRule(rule.rulerRule)) {
-      return;
-    }
-
     try {
-      const ruleGroupId = getRuleGroupLocationFromCombinedRule(rule);
-      const ruleUID = rule.rulerRule.grafana_alert.uid;
-
-      await pauseRule.execute(ruleGroupId, ruleUID, newIsPaused);
+      await pauseRule.execute(groupIdentifier, rule.grafana_alert.uid, newIsPaused);
     } catch (error) {
       notifyApp.error(`Failed to ${newIsPaused ? 'pause' : 'resume'} the rule: ${stringifyErrorLike(error)}`);
       return;
@@ -58,7 +49,7 @@ const MenuItemPauseRule = ({ rule, onPauseChange }: Props) => {
       icon={icon}
       disabled={isLoading(updateState)}
       onClick={() => {
-        setRulePause(!isPaused);
+        setRulePause(!rule.grafana_alert.is_paused);
       }}
     />
   );
