@@ -3,8 +3,16 @@ import { uniqueId } from 'lodash';
 import { FC, useCallback, useState } from 'react';
 import { useFormContext } from 'react-hook-form';
 
-import { DataFrame, dateTimeFormat, GrafanaTheme2, isTimeSeriesFrames, LoadingState, PanelData } from '@grafana/data';
-import { Alert, AutoSizeInput, Button, clearButtonStyles, IconButton, Stack, useStyles2 } from '@grafana/ui';
+import {
+  CoreApp,
+  DataFrame,
+  GrafanaTheme2,
+  LoadingState,
+  PanelData,
+  dateTimeFormat,
+  isTimeSeriesFrames,
+} from '@grafana/data';
+import { Alert, AutoSizeInput, Button, IconButton, Stack, Text, clearButtonStyles, useStyles2 } from '@grafana/ui';
 import { ClassicConditions } from 'app/features/expressions/components/ClassicConditions';
 import { Math } from 'app/features/expressions/components/Math';
 import { Reduce } from 'app/features/expressions/components/Reduce';
@@ -20,6 +28,8 @@ import {
 import { AlertQuery, PromAlertingRuleState } from 'app/types/unified-alerting-dto';
 
 import { usePagination } from '../../hooks/usePagination';
+import { RuleFormValues } from '../../types/rule-form';
+import { isGrafanaRecordingRuleByType } from '../../utils/rules';
 import { PopupCard } from '../HoverCard';
 import { Spacer } from '../Spacer';
 import { AlertStateTag } from '../rules/AlertStateTag';
@@ -58,7 +68,9 @@ export const Expression: FC<ExpressionProps> = ({
 
   const queryType = query?.type;
 
-  const { setError, clearErrors } = useFormContext();
+  const { setError, clearErrors, watch } = useFormContext<RuleFormValues>();
+  const type = watch('type');
+  const isGrafanaRecordingRule = type ? isGrafanaRecordingRuleByType(type) : false;
 
   const onQueriesValidationError = useCallback(
     (errorMsg: string | undefined) => {
@@ -91,7 +103,15 @@ export const Expression: FC<ExpressionProps> = ({
           return <Math onChange={onChangeQuery} query={query} labelWidth={'auto'} onRunQuery={() => {}} />;
 
         case ExpressionQueryType.reduce:
-          return <Reduce onChange={onChangeQuery} refIds={availableRefIds} labelWidth={'auto'} query={query} />;
+          return (
+            <Reduce
+              onChange={onChangeQuery}
+              refIds={availableRefIds}
+              labelWidth={'auto'}
+              app={CoreApp.UnifiedAlerting}
+              query={query}
+            />
+          );
 
         case ExpressionQueryType.resample:
           return <Resample onChange={onChangeQuery} query={query} labelWidth={'auto'} refIds={availableRefIds} />;
@@ -158,20 +178,25 @@ export const Expression: FC<ExpressionProps> = ({
         </div>
         {hasResults && (
           <>
-            <ExpressionResult series={series} isAlertCondition={isAlertCondition} />
+            <ExpressionResult
+              series={series}
+              isAlertCondition={isAlertCondition}
+              isRecordingRule={isGrafanaRecordingRule}
+            />
+            {!isGrafanaRecordingRule && (
+              <div className={styles.footer}>
+                <Stack direction="row" alignItems="center">
+                  <Spacer />
 
-            <div className={styles.footer}>
-              <Stack direction="row" alignItems="center">
-                <Spacer />
-
-                <PreviewSummary
-                  isCondition={Boolean(isAlertCondition)}
-                  firing={groupedByState[PromAlertingRuleState.Firing].length}
-                  normal={groupedByState[PromAlertingRuleState.Inactive].length}
-                  seriesCount={seriesCount}
-                />
-              </Stack>
-            </div>
+                  <PreviewSummary
+                    isCondition={Boolean(isAlertCondition)}
+                    firing={groupedByState[PromAlertingRuleState.Firing].length}
+                    normal={groupedByState[PromAlertingRuleState.Inactive].length}
+                    seriesCount={seriesCount}
+                  />
+                </Stack>
+              </div>
+            )}
           </>
         )}
       </div>
@@ -182,9 +207,10 @@ export const Expression: FC<ExpressionProps> = ({
 interface ExpressionResultProps {
   series: DataFrame[];
   isAlertCondition?: boolean;
+  isRecordingRule?: boolean;
 }
 export const PAGE_SIZE = 20;
-export const ExpressionResult: FC<ExpressionResultProps> = ({ series, isAlertCondition }) => {
+export const ExpressionResult: FC<ExpressionResultProps> = ({ series, isAlertCondition, isRecordingRule = false }) => {
   const { pageItems, previousPage, nextPage, numberOfPages, pageStart, pageEnd } = usePagination(series, 1, PAGE_SIZE);
   const styles = useStyles2(getStyles);
 
@@ -212,7 +238,13 @@ export const ExpressionResult: FC<ExpressionResultProps> = ({ series, isAlertCon
         !isTimeSeriesResults &&
         pageItems.map((frame, index) => (
           // There's no way to uniquely identify a frame that doesn't cause render bugs :/ (Gilles)
-          <FrameRow key={uniqueId()} frame={frame} index={pageStart + index} isAlertCondition={isAlertCondition} />
+          <FrameRow
+            key={uniqueId()}
+            frame={frame}
+            index={pageStart + index}
+            isAlertCondition={isAlertCondition}
+            isRecordingRule={isRecordingRule}
+          />
         ))}
       {emptyResults && <div className={cx(styles.expression.noData, styles.mutedText)}>No data</div>}
       {shouldShowPagination && (
@@ -337,13 +369,17 @@ const Header: FC<HeaderProps> = ({
           <div>{getExpressionLabel(queryType)}</div>
         </Stack>
         <Spacer />
-        <ExpressionStatusIndicator onSetCondition={() => onSetCondition(query.refId)} isCondition={alertCondition} />
+        <ExpressionStatusIndicator
+          refId={refId}
+          onSetCondition={() => onSetCondition(query.refId)}
+          isCondition={alertCondition}
+        />
         <IconButton
           name="trash-alt"
           variant="secondary"
           className={styles.mutedIcon}
           onClick={onRemoveExpression}
-          tooltip="Remove expression"
+          tooltip={`Remove expression "${refId}"`}
         />
       </Stack>
     </header>
@@ -353,9 +389,16 @@ const Header: FC<HeaderProps> = ({
 interface FrameProps extends Pick<ExpressionProps, 'isAlertCondition'> {
   frame: DataFrame;
   index: number;
+  isRecordingRule?: boolean;
 }
 
-const FrameRow: FC<FrameProps> = ({ frame, index, isAlertCondition }) => {
+const OpeningBracket = () => <span>{'{'}</span>;
+const ClosingBracket = () => <span>{'}'}</span>;
+// eslint-disable-next-line @grafana/no-untranslated-strings
+const Quote = () => <span>&quot;</span>;
+const Equals = () => <span>{'='}</span>;
+
+function FrameRow({ frame, index, isAlertCondition, isRecordingRule }: FrameProps) {
   const styles = useStyles2(getStyles);
 
   const name = getSeriesName(frame) || 'Series ' + index;
@@ -368,38 +411,46 @@ const FrameRow: FC<FrameProps> = ({ frame, index, isAlertCondition }) => {
   const showNormal = isAlertCondition && value === 0;
 
   const title = `${hasLabels ? '' : name}${hasLabels ? `{${formatLabels(labelsRecord)}}` : ''}`;
+  const shouldRenderSumary = !isRecordingRule;
 
   return (
     <div className={styles.expression.resultsRow}>
       <Stack direction="row" gap={1} alignItems="center">
         <div className={styles.expression.resultLabel} title={title}>
-          <span>{hasLabels ? '' : name}</span>
-          {hasLabels && (
-            <>
-              <span>{'{'}</span>
-              {labels.map(([key, value], index) => (
-                <span key={uniqueId()}>
-                  <span className={styles.expression.labelKey}>{key}</span>
-                  <span>=</span>
-                  <span>&quot;</span>
-                  <span className={styles.expression.labelValue}>{value}</span>
-                  <span>&quot;</span>
-                  {index < labels.length - 1 && <span>, </span>}
-                </span>
-              ))}
-              <span>{'}'}</span>
-            </>
-          )}
+          <Text variant="code">
+            {hasLabels ? (
+              <>
+                <OpeningBracket />
+                {labels.map(([key, value], index) => (
+                  <Text variant="body" key={uniqueId()}>
+                    <span className={styles.expression.labelKey}>{key}</span>
+                    <Equals />
+                    <Quote />
+                    <span className={styles.expression.labelValue}>{value}</span>
+                    <Quote />
+                    {index < labels.length - 1 && <span>, </span>}
+                  </Text>
+                ))}
+                <ClosingBracket />
+              </>
+            ) : (
+              <span className={styles.expression.labelKey}>{title}</span>
+            )}
+          </Text>
         </div>
         <div className={styles.expression.resultValue}>{value}</div>
-        {showFiring && <AlertStateTag state={PromAlertingRuleState.Firing} size="sm" />}
-        {showNormal && <AlertStateTag state={PromAlertingRuleState.Inactive} size="sm" />}
+        {shouldRenderSumary && (
+          <>
+            {showFiring && <AlertStateTag state={PromAlertingRuleState.Firing} size="sm" />}
+            {showNormal && <AlertStateTag state={PromAlertingRuleState.Inactive} size="sm" />}
+          </>
+        )}
       </Stack>
     </div>
   );
-};
-
-const TimeseriesRow: FC<FrameProps & { index: number }> = ({ frame, index }) => {
+}
+interface TimeseriesRowProps extends Omit<FrameProps, 'isRecordingRule'> {}
+const TimeseriesRow: FC<TimeseriesRowProps & { index: number }> = ({ frame, index }) => {
   const styles = useStyles2(getStyles);
 
   const valueField = frame.fields[1]; // field 0 is "time", field 1 is "value"
