@@ -1,4 +1,5 @@
 import { Tooltip, Button, Stack } from '@grafana/ui';
+import { DashboardInteractions } from 'app/features/dashboard-scene/utils/interactions';
 
 type VersionsButtonsType = {
   hasMore: boolean;
@@ -16,7 +17,15 @@ export const VersionsHistoryButtons = ({
 }: VersionsButtonsType) => (
   <Stack>
     {hasMore && (
-      <Button type="button" onClick={() => getVersions(true)} variant="secondary" disabled={isLastPage}>
+      <Button
+        type="button"
+        onClick={() => {
+          getVersions(true);
+          DashboardInteractions.showMoreVersionsClicked();
+        }}
+        variant="secondary"
+        disabled={isLastPage}
+      >
         Show more versions
       </Button>
     )}
